{ :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  :: Bugs Free QuickReport 3.0 for Delphi and C++Builder     ::
  ::                                                         ::
  :: QRDlg.PAS - Page Setup Dialog                           ::
  ::                                                         ::
  :: Copyright (c) 2000 Bugs Free QR Team                    ::
  ::                                                         ::
  :: web: http://bfqr.nm.ru                                  ::
  ::                                                         ::
  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

{$I QRDEFS.INC}
{$IFDEF BFQR_D6}
  {$WARN SYMBOL_PLATFORM OFF}  //RAA
{$ENDIF}

unit QrDlg;
{$R-,T-,H+,X+}

interface

uses Windows, Messages, SysUtils, CommDlg, Classes, Graphics, Controls,
     Forms, Printers;
type

{ TrCommonDialog }

  TQRCommonDialog = class(TComponent)
  private
    FCtl3D: Boolean;
    FDefWndProc: Pointer;
    FHelpContext: THelpContext;
    FHandle: HWnd;
    FObjectInstance: Pointer;
    FTemplate: PChar;
    FOnClose: TNotifyEvent;
    FOnShow: TNotifyEvent;

    FDeltaWidth,
    FDeltaHeight: Integer;

    procedure WMDestroy(var Message: TWMDestroy); message WM_DESTROY;
    procedure WMInitDialog(var Message: TWMInitDialog); message WM_INITDIALOG;
    procedure WMNCDestroy(var Message: TWMNCDestroy); message WM_NCDESTROY;
    procedure MainWndProc(var Message: TMessage);
  protected
    procedure DoClose; dynamic;
    procedure DoShow; dynamic;
    procedure WndProc(var Message: TMessage); virtual;
    function MessageHook(var Msg: TMessage): Boolean; virtual;
    function TaskModalDialog(DialogFunc: Pointer; var DialogData): Bool; virtual;
    function Execute: Boolean; virtual; abstract;
    property Template: PChar read FTemplate write FTemplate;

    property DeltaWidth: Integer read FDeltaWidth write FDeltaWidth;
    property DeltaHeight: Integer read FDeltaHeight write FDeltaHeight;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DefaultHandler(var Message); override;
    property Handle: HWnd read FHandle;
  published
    property Ctl3D: Boolean read FCtl3D write FCtl3D default True;
    property HelpContext: THelpContext read FHelpContext write FHelpContext default 0;
    property OnClose: TNotifyEvent read FOnClose write FOnClose;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;
  end;

{TPageSetupDialog}
  TQRPageSetupDialog = class;
  TPageSetupFlags = (poDefaultMinMargins, poDisableMargins, poDisableOrientation,
                     poDisablePagePainting, poDisablePaper, poDisablePrinter,
                     poNoWarning, poShowHelp);
  TPageOptions    = Set of TPageSetupFlags;

  TPaperType = (ptPaper, ptEnvelope);
  TPaperOrientation = TPrinterOrientation;
  TPrinterType = (ptDotMatrix, ptHPPCL);
  TPaintWhat = (pwFullPage, pwMinimumMargins, pwMargins,
                pwGreekText, pwEnvStamp, pwYAFullPage);

  TMeasureVal = Double;
  TMeasurements = (pmDefault, pmMillimeters, pmInches);

  TPageSetupRect = class(TPersistent)
  private
    FOwner: TQRPageSetupDialog;
    FRect: TRect;
    function GetAbsRect(const Index: Integer): Integer;
    procedure SetAbsRect(const Index, Value: Integer);

    function GetRect(const Index: Integer): Double;
    procedure SetRect(const Index: Integer; Value: Double);

  public
    constructor Create(aOwner: TQRPageSetupDialog); virtual;
    destructor Destroy; override;
    function Compare(Other: TPageSetupRect): boolean;
    property Rect:      TRect           read FRect       write FRect;
    property absLeft:   Integer index 0 read  GetAbsRect write SetAbsRect;
    property absRight:  Integer index 1 read  GetAbsRect write SetAbsRect;
    property absTop:    Integer index 2 read  GetAbsRect write SetAbsRect;
    property absBottom: Integer index 3 read  GetAbsRect write SetAbsRect;
  published
    property Left:   Double index 0 read  GetRect write SetRect;
    property Right:  Double index 1 read  GetRect write SetRect;
    property Top:    Double index 2 read  GetRect write SetRect;
    property Bottom: Double index 3 read  GetRect write SetRect;
  end;

  TPageSetupPoint = class(TPersistent)
  private
    FOwner: TQRPageSetupDialog;
    FPoint: TPoint;
    function GetAbsXY(const Index: Integer): Longint;
    procedure SetAbsXY(const Index: Integer; Value: Longint);
    function GetXY(const Index: Integer): Double;
    procedure SetXY(const Index: Integer; Value: Double);
  public
    constructor Create(aOwner: TQRPageSetupDialog); virtual;
    destructor Destroy; override;
    function Compare(Other: TPageSetupPoint): boolean;
    property Point: TPoint          read FPoint   write FPoint;
    property absX:  longint index 0 read GetAbsXY write SetAbsXY;
    property absY:  longint index 1 read GetAbsXY write SetAbsXY;
  published
    property X: Double index 0 read GetXY write SetXY;
    property Y: Double index 1 read GetXY write SetXY;
  end;

  TQRPageSetupDialog = class(TQRCommonDialog)
  private
    FOptions : TPageOptions;
    FFlags   : DWORD;

    FDefPaperSize,
    FPaperSize: TPageSetupPoint;
    FDefMinMargin,
    FDefMargin,
    FMinMargin,
    FMargin: TPageSetupRect;
    FDefdmPaperSize,
    FdmPaperSize: SmallInt;

    FPrinter: TPrinter;
    FDefOrientation,
    FOrientation: TPaperOrientation;

    FMeasurements: TMeasurements;

    procedure SetMargin(Value: TPageSetupRect);
    procedure SetMinMargin(Value: TPageSetupRect);
    procedure SetPageSize(Value: TPageSetupPoint);
    procedure SetPrinter(Value: TPrinter);
    procedure SetMeasurements(Value: TMeasurements);

    procedure SetOptions(Value : TPageOptions);
    function GetDefaultMeasurements: TMeasurements;
    procedure SetdmPaperSize(Value: SmallInt);
  protected
    function DoExecute(Func: Pointer): Bool;
    procedure _GetPrinter(var DeviceMode, DeviceNames: THandle);
    procedure CreateFlags;

    function StoredMeasurements: boolean;

  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    function Execute: Boolean; override;
    procedure GetDefaults;
    property DefdmPaperSize:  SmallInt          read FDefdmPaperSize;
    property DefMargin:       TPageSetupRect    read FDefMargin;
    property DefMinMargin:    TPageSetupRect    read FDefMinMargin;
    property DefPaperSize:    TPageSetupPoint   read FDefPaperSize;
    property DefOrientation:  TPaperOrientation read FDefOrientation;
    property DefMeasurements: TMeasurements     read GetDefaultMeasurements;
    property Printer:         TPrinter          read FPrinter     write SetPrinter;
    property dmPaperSize:     SmallInt          read FdmPaperSize write SetdmPaperSize;

  published
    property Options:      TPageOptions      read FOptions      write SetOptions      default [poDefaultMinMargins, poShowHelp];
    property Margin:       TPageSetupRect    read FMargin       write SetMargin;
    property MinMargin:    TPageSetupRect    read FMinMargin    write SetMinMargin;
    property PaperSize:    TPageSetupPoint   read FPaperSize    write SetPageSize;
    property Orientation:  TPaperOrientation read FOrientation  write FOrientation    default poPortrait;
    property Measurements: TMeasurements     read FMeasurements write SetMeasurements stored StoredMeasurements;
  end;


  TQRColumnPageSetupDialog = class(TQRPageSetupDialog)
  private

    FColumns: Integer;
    FColumnSpace: Extended;
    procedure SetColumns(Value: Integer);
    procedure SetColumnSpace(Value: Extended);
  protected
    procedure DoShow; override;
//    procedure DoColumn; virtual;
    function ColumnSpaceStore: Boolean;
  public
    function Execute: boolean; override;
    property Columns: Integer read FColumns write SetColumns default 1;
    property ColumnSpace: Extended read FColumnSpace write SetColumnSpace stored ColumnSpaceStore;
  end;

function DefaultMeasurements: TMeasurements;

implementation

{ TSK uses QRColEd; }
{ Private globals }

{$R qrdlg.res}

const
  MeasurementsDiv : array [pmMillimeters..pmInches] of Double = (100.0,1000.0);

var
  CreationControl: TQRCommonDialog = nil;
  HelpMsg: Cardinal;
  WndProcPtrAtom: TAtom = 0;

{ Center the given window on the screen }

function DefaultMeasurements: TMeasurements;
begin
  if GetLocaleChar(LOCALE_USER_DEFAULT,LOCALE_IMEASURE,'0') = '0'
  then Result:= pmMillimeters
  else Result:= pmInches;
end;

procedure CenterWindow(Wnd: HWnd; DW, DH: Integer);
var
  Rect: TRect;
  Monitor: TMonitor;
  W, H: Integer;
begin
  GetWindowRect(Wnd, Rect);
  if Application.MainForm <> nil then
    Monitor := Application.MainForm.Monitor
  else
    Monitor := Screen.Monitors[0];
  W := Rect.Right-Rect.Left + DW;
  H := Rect.Bottom-Rect.Top + DH;
  SetWindowPos(Wnd, 0,
    Monitor.Left + ((Monitor.Width - Rect.Right + Rect.Left) div 2),
    Monitor.Top + ((Monitor.Height - Rect.Bottom + Rect.Top) div 3),
    W, H, SWP_NOACTIVATE {or SWP_NOSIZE} or SWP_NOZORDER);
end;

{ Generic dialog hook. Centers the dialog on the screen in response to
  the WM_INITDIALOG message }

type
  PTagCreateStruct = ^TagCreateStruct;

function DialogHook(Wnd: HWnd; Msg: UINT; WParam: WPARAM; LParam: LPARAM): UINT; stdcall;
begin
  Result := 0;
  case Msg of
    WM_INITDIALOG: begin
      CenterWindow(Wnd, CreationControl.DeltaWidth, CreationControl.DeltaHeight);
      CreationControl.FHandle := Wnd;
      CreationControl.FDefWndProc := Pointer(SetWindowLong(Wnd, GWL_WNDPROC,
        Longint(CreationControl.FObjectInstance)));
      CallWindowProc(CreationControl.FObjectInstance, Wnd, Msg, WParam, LParam);
    end;
  end
end;

const
  ColumnCode=501;

var
  ColumnDialog: TQRColumnPageSetupDialog;

function PageDialogHook(Wnd: HWnd; Msg: UINT; WParam: WPARAM; LParam: LPARAM): UINT; stdcall;
begin
  if (Msg = WM_COMMAND)
  and (LongRec(WParam).Hi = BN_CLICKED)
  and (LongRec(WParam).Lo = ColumnCode)
  then begin
    //        
{TSK    ColumnDialog.DoColumn;}
    Result :=1;
  end
  else Result := DialogHook(Wnd, Msg, WParam, LParam);
end;


{ TQRCommonDialog }

constructor TQRCommonDialog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCtl3D := True;
  FObjectInstance := {$IFDEF BFQR_D6}Classes.{$ENDIF}MakeObjectInstance(MainWndProc);
  FDeltaWidth := 0;
  FDeltaHeight:= 0;
end;

destructor TQRCommonDialog.Destroy;
begin
  if FObjectInstance <> nil then {$IFDEF BFQR_D6}Classes.{$ENDIF}FreeObjectInstance(FObjectInstance);
  inherited Destroy;
end;

function TQRCommonDialog.MessageHook(var Msg: TMessage): Boolean;
begin
  Result := False;
  if (Msg.Msg = HelpMsg) and (FHelpContext <> 0) then
  begin
    Application.HelpContext(FHelpContext);
    Result := True;
  end;
end;

procedure TQRCommonDialog.DefaultHandler(var Message);
begin
  if FHandle <> 0 then
    with TMessage(Message) do
      Result := CallWindowProc(FDefWndProc, FHandle, Msg, WParam, LParam)
  else inherited DefaultHandler(Message);
end;

procedure TQRCommonDialog.MainWndProc(var Message: TMessage);
begin
  try
    WndProc(Message);
  except
    Application.HandleException(Self);
  end;
end;

procedure TQRCommonDialog.WndProc(var Message: TMessage);
begin
  Dispatch(Message);
end;

procedure TQRCommonDialog.WMDestroy(var Message: TWMDestroy);
begin
  inherited;
  DoClose;
end;

procedure TQRCommonDialog.WMInitDialog(var Message: TWMInitDialog);
begin
  { Called only by non-explorer style dialogs }
  DoShow;
  { Prevent any further processing }
  Message.Result := 0;
end;

procedure TQRCommonDialog.WMNCDestroy(var Message: TWMNCDestroy);
begin
  inherited;
  FHandle := 0;
end;

function TQRCommonDialog.TaskModalDialog(DialogFunc: Pointer; var DialogData): Bool;
type
  TDialogFunc = function(var DialogData): Bool stdcall;
var
  ActiveWindow: HWnd;
  WindowList: Pointer;
  FPUControlWord: Word;
begin
  ActiveWindow := GetActiveWindow;
  WindowList := DisableTaskWindows(0);
  try
    Application.HookMainWindow(MessageHook);
    asm
      // Avoid FPU control word change in NETRAP.dll, NETAPI32.dll, etc
      FNSTCW  FPUControlWord
    end;
    try
      CreationControl := Self;
      Result := TDialogFunc(DialogFunc)(DialogData);
    finally
      asm
        FNCLEX
        FLDCW FPUControlWord
      end;
      Application.UnhookMainWindow(MessageHook);
    end;
  finally
    EnableTaskWindows(WindowList);
    SetActiveWindow(ActiveWindow);
  end;
end;

procedure TQRCommonDialog.DoClose;
begin
  if Assigned(FOnClose) then FOnClose(Self);
end;

procedure TQRCommonDialog.DoShow;
begin
  if Assigned(FOnShow) then FOnShow(Self);
end;

{ TPageSetupRect }

constructor TPageSetupRect.Create(aOwner: TQRPageSetupDialog);
begin
  inherited Create;
  FOwner := aOwner;
  FillChar(FRect, SizeOf(TRect),0);
end;

destructor TPageSetupRect.Destroy;
begin
  FOwner := nil;
  inherited Destroy;
end;

function TPageSetupRect.Compare(Other: TPageSetupRect): boolean;
begin
  Result := EqualRect(Rect, Other.Rect);
end;

function TPageSetupRect.GetAbsRect(const Index: Integer): Integer;
begin
  case Index of
  0: Result := FRect.Left;
  1: Result := FRect.Right;
  2: Result := FRect.Top;
  3: Result := FRect.Bottom;
  else Result := 0;
  end;
end;

procedure TPageSetupRect.SetAbsRect(const Index, Value: Integer);
begin
  if GetRect(Index) <> Value
  then case Index of
    0: FRect.Left   := Value;
    1: FRect.Right  := Value;
    2: FRect.Top    := Value;
    3: FRect.Bottom := Value;
    end;
end;

function TPageSetupRect.GetRect(const Index: Integer): Double;
var
  Tmp: TMeasurements;
begin
  Tmp := DefaultMeasurements;
  if Assigned(FOwner)
  then Tmp := FOwner.Measurements;
  case Index of
  0: Result := FRect.Left / MeasurementsDiv[Tmp];
  1: Result := FRect.Right / MeasurementsDiv[Tmp];
  2: Result := FRect.Top / MeasurementsDiv[Tmp];
  3: Result := FRect.Bottom / MeasurementsDiv[Tmp];
  else Result := 0;
  end;
end;

procedure TPageSetupRect.SetRect(const Index: Integer; Value: Double);
var
  Tmp: TMeasurements;
  RValue: Integer;
begin
  Tmp := DefaultMeasurements;
  if Assigned(FOwner)
  then Tmp := FOwner.Measurements;
  RValue := Round(Value*MeasurementsDiv[Tmp]);
  if GetRect(Index) <> RValue
  then case Index of
    0: FRect.Left   := RValue;
    1: FRect.Right  := RValue;
    2: FRect.Top    := RValue;
    3: FRect.Bottom := RValue;
    end;
end;

{TPageSetupPoint}

constructor TPageSetupPoint.Create(aOwner: TQRPageSetupDialog);
begin
  inherited Create;
  FOwner := aOwner;
  FillChar(FPoint, SizeOf(TPoint), 0);
end;

destructor TPageSetupPoint.Destroy;
begin
  FOwner := nil;
  inherited Destroy;
end;

function TPageSetupPoint.Compare(Other: TPageSetupPoint): boolean;
begin
  Result := (FPoint.X = Other.Point.X) and (FPoint.Y = Other.Point.Y);
end;

function TPageSetupPoint.GetAbsXY(const Index: Integer): Longint;
begin
  case Index of
  0: Result := FPoint.X;
  1: Result := FPoint.Y;
  else Result := 0;
  end;
end;

procedure TPageSetupPoint.SetAbsXY(const Index: Integer; Value: Longint);
begin
  if GetXY(Index) <> Value then
    case Index of
    0: FPoint.X := Value;
    1: FPoint.Y := Value;
    end;
end;

function TPageSetupPoint.GetXY(const Index: Integer): Double;
var
  Tmp: TMeasurements;
begin
  Tmp := DefaultMeasurements;
  if Assigned(FOwner)
  then Tmp := FOwner.Measurements;
  case Index of
  0: Result := FPoint.X / MeasurementsDiv[Tmp];
  1: Result := FPoint.Y / MeasurementsDiv[Tmp];
  else Result := 0;
  end;
end;

procedure TPageSetupPoint.SetXY(const Index: Integer; Value: Double);
var
  Tmp: TMeasurements;
  RValue: Integer;
begin
  Tmp := DefaultMeasurements;
  if Assigned(FOwner)
  then Tmp := FOwner.Measurements;
  RValue := Round(Value*MeasurementsDiv[Tmp]);
  if GetXY(Index) <> RValue then
    case Index of
    0: FPoint.X := RValue;
    1: FPoint.Y := RValue;
    end;
end;

{TQRPageSetupDialog}

procedure Set_Printer(DeviceMode, DeviceNames: THandle);
var
  DevNames: PDevNames;
begin
  DevNames := PDevNames(GlobalLock(DeviceNames));
  try
    with DevNames^ do
      Printer.SetPrinter(PChar(DevNames) + wDeviceOffset, PChar(DevNames) +
         wDriverOffset, PChar(DevNames) + wOutputOffset, DeviceMode);
  finally
    GlobalUnlock(DeviceNames);
    GlobalFree(DeviceNames);
  end;
end;

function CopyData(Handle: THandle): THandle;
var
  Src, Dest: PChar;
  Size: Integer;
begin
  if Handle <> 0 then
  begin
    Size := GlobalSize(Handle);
    Result := GlobalAlloc(GHND, Size);
    if Result <> 0 then
      try
        Src := GlobalLock(Handle);
        Dest := GlobalLock(Result);
        if (Src <> nil) and (Dest <> nil) then Move(Src^, Dest^, Size);
      finally
        GlobalUnlock(Handle);
        GlobalUnlock(Result);
      end
  end
  else Result := 0;
end;

constructor TQRPageSetupDialog.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FPrinter := Printers.Printer;
  Options := [ poShowHelp];
  FMargin := TPageSetupRect.Create(Self);
  FMinMargin := TPageSetupRect.Create(Self);
  FPaperSize := TPageSetupPoint.Create(Self);
  FDefMargin := TPageSetupRect.Create(Self);
  FDefMinMargin := TPageSetupRect.Create(Self);
  FDefPaperSize := TPageSetupPoint.Create(Self);
  FMeasurements := DefMeasurements;
  GetDefaults;
  FOrientation := FDefOrientation;
  FdmPaperSize := FDefdmPaperSize;
end;

destructor TQRPageSetupDialog.Destroy;
begin
  FPaperSize.Free;
  FMinMargin.Free;
  FMargin.Free;
  FDefPaperSize.Free;
  FDefMinMargin.Free;
  FDefMargin.Free;
  inherited Destroy;
end;

procedure TQRPageSetupDialog._GetPrinter(var DeviceMode, DeviceNames: THandle);
var
  Device, Driver, Port: array[0..79] of char;
  DevNames: PDevNames;
  Offset: PChar;
begin
  if Assigned(FPrinter) then begin
    FPrinter.GetPrinter(Device, Driver, Port, DeviceMode);
    if DeviceMode <> 0 then
    begin
      DeviceNames := GlobalAlloc(GHND, SizeOf(TDevNames) +
       StrLen(Device) + StrLen(Driver) + StrLen(Port) + 3);
      DevNames := PDevNames(GlobalLock(DeviceNames));
      try
        Offset := PChar(DevNames) + SizeOf(TDevnames);
        with DevNames^ do
        begin
          wDriverOffset := Longint(Offset) - Longint(DevNames);
          Offset := StrECopy(Offset, Driver) + 1;
          wDeviceOffset := Longint(Offset) - Longint(DevNames);
          Offset := StrECopy(Offset, Device) + 1;
          wOutputOffset := Longint(Offset) - Longint(DevNames);;
          StrCopy(Offset, Port);
        end;
      finally
        GlobalUnlock(DeviceNames);
      end;
    end;
  end;
end;

function TQRPageSetupDialog.DoExecute(Func: Pointer): Bool;
var
  PageDlgRec: TPageSetupDlg;
  DevHandle: THandle;
  PDevMode: PDeviceMode;

begin
  FillChar(PageDlgRec, SizeOf(PageDlgRec), 0);
  with PageDlgRec do
  try
    lStructSize := SizeOf(PageDlgRec);
    PageDlgRec.hInstance := SysInit.HInstance;

    CreateFlags;
    Flags       := PSD_ENABLEPAGESETUPHOOK or FFlags or PSD_MARGINS;
    lpfnPageSetupHook := PageDialogHook;

    rtMargin    := FMargin.Rect;

    if poDefaultMinMargins in FOptions
    then rtMinMargin := FDefMinMargin.Rect
    else rtMinMargin := FMinMargin.Rect;

    if (Flags and PSD_MINMARGINS) <> 0 then begin
      if rtMinMargin.Left < FDefMinMargin.AbsLeft     then rtMinMargin.Left   := FDefMinMargin.AbsLeft;
      if rtMinMargin.Right < FDefMinMargin.AbsRight   then rtMinMargin.Right  := FDefMinMargin.AbsRight;
      if rtMinMargin.Top < FDefMinMargin.AbsTop       then rtMinMargin.Top    := FDefMinMargin.AbsTop;
      if rtMinMargin.Bottom < FDefMinMargin.AbsBottom then rtMinMargin.Bottom := FDefMinMargin.AbsBottom;
    end;
    ptPaperSize := FPaperSize.Point;

    _GetPrinter(DevHandle,hDevNames);
    hDevMode    := CopyData(DevHandle);

    if (Flags and PSD_MINMARGINS) <> 0 then begin
      if rtMargin.Left < rtMinMargin.Left then rtMargin.Left := rtMinMargin.Left;
      if rtMargin.Right < rtMinMargin.Right then rtMargin.Right := rtMinMargin.Right;
      if rtMargin.Top < rtMinMargin.Top then rtMargin.Top := rtMinMargin.Top;
      if rtMargin.Bottom < rtMinMargin.Bottom then rtMargin.Bottom := rtMinMargin.Bottom;
    end;

    try
      PDevMode := GlobalLock(HDevMode);
      if pcOrientation in FPrinter.Capabilities then
        PDevMode.dmOrientation := ord(FOrientation)+1;
      PDevMode.dmPaperSize := FdmPaperSize;
    finally
      GlobalUnlock(hDevMode);
    end;

    hWndOwner   := Application.Handle;

    if Template <> nil then begin
      lpPageSetupTemplateName := Template;
      Flags := Flags or PSD_ENABLEPAGESETUPTEMPLATE;
    end;

    Result := TaskModalDialog(Func, PageDlgRec);
    if Result then begin
      FMargin.Rect := rtMargin;
      FMinMargin.Rect := rtMinMargin;
      FPaperSize.Point := ptPaperSize;
      try
        PDevMode := GlobalLock(HDevMode);
        if pcOrientation in FPrinter.Capabilities then
          FOrientation := TPaperOrientation(PDevMode.dmOrientation-1);
        dmPaperSize := PDevMode.dmPaperSize;
      finally
        GlobalUnlock(hDevMode);
      end;
      Set_Printer(DevHandle,hDevNames);
    end;
    if hDevMode <> 0 then GlobalFree(hDevMode);
    if hDevNames <> 0 then GlobalFree(hDevNames);
  finally
  end;
end;

function TQRPageSetupDialog.Execute: Boolean;
begin
  Result := DoExecute(@PageSetupDlg);
end;

procedure TQRPageSetupDialog.GetDefaults;
const
  PageSetupMeasurements: array [TMeasurements] of DWord = (
     0, PSD_INHUNDREDTHSOFMILLIMETERS, PSD_INTHOUSANDTHSOFINCHES
    );
var
  PageDlgRec: TPageSetupDlg;
  PDevMode: PDeviceMode;

begin
  FillChar(PageDlgRec, SizeOf(PageDlgRec), 0);
  with PageDlgRec do
  try
    lStructSize := SizeOf(PageDlgRec);

    PageDlgRec.hInstance := SysInit.hInstance;

    hWndOwner   := Application.Handle;
    Flags :=  PSD_MARGINS or PSD_DEFAULTMINMARGINS
         or PSD_INTHOUSANDTHSOFINCHES
         or PSD_RETURNDEFAULT
         or PSD_NOWARNING;

    PageSetupDlg(PageDlgRec);

    FDefMargin.Rect  := rtMargin;
    FDefMinMargin.Rect := rtMinMargin;
    FDefPaperSize.Point := ptPaperSize;
    if pcOrientation in FPrinter.Capabilities then
    try
      PDevMode := GlobalLock(PageDlgRec.hDevMode);
      if Assigned(PDevMode) then begin
        FDefdmPaperSize := PDevMode.dmPaperSize;
        FDefOrientation := TPaperOrientation(PDevMode.dmOrientation-1)
      end
      else begin
        FDefOrientation := FPrinter.Orientation;
        FDefdmPaperSize := 0;
      end;
    finally
      GlobalUnlock(PageDlgRec.hDevMode);
    end;
    if DefMeasurements = pmMillimeters then begin
      FDefMargin.Rect := Rect( muldiv(FDefMargin.Rect.Left,254,100),
                               muldiv(FDefMargin.Rect.Top,254,100),
                               muldiv(FDefMargin.Rect.Right,254,100),
                               muldiv(FDefMargin.Rect.Bottom,254,100));
      FDefMinMargin.Rect := Rect( muldiv(FDefMinMargin.Rect.Left,254,100),
                                  muldiv(FDefMinMargin.Rect.Right,254,100),
                                  muldiv(FDefMinMargin.Rect.Top,254,100),
                                  muldiv(FDefMinMargin.Rect.Bottom,254,100));
      FDefPaperSize.Point := Point( muldiv(FDefPaperSize.Point.x,254,100),
                                    muldiv(FDefPaperSize.Point.y,254,100));
    end;

    if hDevMode <> 0 then GlobalFree(hDevMode);
    if hDevNames <> 0 then GlobalFree(hDevNames);
  finally
  end;
end;

function TQRPageSetupDialog.GetDefaultMeasurements: TMeasurements;
begin
  Result := DefaultMeasurements;
end;

procedure TQRPageSetupDialog.CreateFlags;
const
  PageSetupOptions: array [TPageSetupFlags] of DWord = (
     PSD_DEFAULTMINMARGINS, PSD_DISABLEMARGINS, PSD_DISABLEORIENTATION,
     PSD_DISABLEPAGEPAINTING, PSD_DISABLEPAPER, PSD_DISABLEPRINTER,
     PSD_NOWARNING, PSD_SHOWHELP
    );
  PageSetupMeasurements: array [TMeasurements] of DWord = (
     0, PSD_INHUNDREDTHSOFMILLIMETERS, PSD_INTHOUSANDTHSOFINCHES
    );
var
  Option: TPageSetupFlags;
begin
  FFlags := PageSetupMeasurements[FMeasurements];
  for Option := Low(Option) to High(Option) do
    if Option in FOptions
    then FFlags := FFlags OR PageSetupOptions[Option];
  if not (poDefaultMinMargins in FOptions)
  then FFlags := FFlags OR PSD_MINMARGINS;
end;

function TQRPageSetupDialog.StoredMeasurements: boolean;
begin
  Result := DefaultMeasurements <> FMeasurements;
end;

procedure TQRPageSetupDialog.SetMeasurements(Value: TMeasurements);
begin
  FMeasurements := Value;
  CreateFlags;
end;                   

procedure TQRPageSetupDialog.SetOptions(Value : TPageOptions);

begin
  FOptions := Value;
  CreateFlags;
end;

procedure TQRPageSetupDialog.SetMargin(Value: TPageSetupRect);
begin
  FMargin.Rect := Value.Rect;
end;

procedure TQRPageSetupDialog.SetMinMargin(Value: TPageSetupRect);
begin
  FMinMargin.Rect := Value.Rect;
end;

procedure TQRPageSetupDialog.SetPageSize(Value: TPageSetupPoint);
begin
  FPaperSize.Point := Value.Point;
end;

procedure TQRPageSetupDialog.SetPrinter(Value: TPrinter);
begin
  if Value <> FPrinter
  then begin
    FPrinter := Value;
    if FPrinter = nil then FPrinter := Printers.Printer;
    GetDefaults;
  end;
end;

procedure TQRPageSetupDialog.SetdmPaperSize(Value: SmallInt);
begin
  if ((Value >= 0) and (Value <= dmpaper_USER)) and (Value <> FdmPaperSize)
  then FdmPaperSize := Value;
end;

{ TQRColumnPageSetupDialog }

{TSKprocedure TQRColumnPageSetupDialog.DoColumn;
begin
  with TQRColumnEditor.Create(Self) do try
    Measurements := ord(FMeasurements);
    Columns := FColumns;
    ColumnSpace := FColumnSpace;
    if ShowModal = mrOK then begin
      FColumns := Columns;
      FColumnSpace := ColumnSpace;
    end;
  finally
    Free;
  end
end;}

procedure TQRColumnPageSetupDialog.DoShow;
begin
  inherited DoShow;
  SendMessage(Handle, WM_SETFOCUS, 0, 0);
end;


procedure TQRColumnPageSetupDialog.SetColumns(Value: Integer);
begin
  if (Value >= 1) and (FColumns <> Value)
  then FColumns := Value;
end;

procedure TQRColumnPageSetupDialog.SetColumnSpace(Value: Extended);
begin
  if (Value >= 0) and (FColumnSpace <> Value)
  then FColumnSpace := Value;
end;

function TQRColumnPageSetupDialog.ColumnSpaceStore: Boolean;
begin
  Result := FColumnSpace <> 0;
end;

function TQRColumnPageSetupDialog.Execute: boolean;
begin
  try
    ColumnDialog := Self;
    Template := 'PGSET';
    Result := inherited Execute;
  finally
    ColumnDialog := nil;
  end;
end;

{ Initialization and cleanup }

procedure InitGlobals;
var
  AtomText: array[0..31] of Char;
begin
  HelpMsg := RegisterWindowMessage(HelpMsgString);
  WndProcPtrAtom := GlobalAddAtom(StrFmt(AtomText,
    'WndProcPtr%.8X%.8X', [HInstance, GetCurrentThreadID]));
end;

initialization
  InitGlobals;
finalization
  if WndProcPtrAtom <> 0 then GlobalDeleteAtom(WndProcPtrAtom);
end.

