{===================================================================================================

  Comming Library

    DBEdit,  .
        RxLib.  .    (*...*).

====================================================================================================

  :

  29.11.2002 [*]    CommingDesign  CommingRuntime
  25.10.2002 [-] .
  15.10.2002 [+]  -.
  12.09.2002 [*]  ().
  15.12.2001 [*] ReadOnly:  .
  14.12.2001 [*] TcoDBDateEdit:    dd.mm.yyyy.
  13.12.2001 [+] :  OnCheck,  .
  16.08.2001 [-]   $IFDEF.
  14.08.2001 [+]  .
  09.09.1998 [!]  Comming Library.

====================================================================================================
  Copyright 19982007 TSK Developer Laboratory, Moscow.                     Konstantin Tsvetkov
                                                                              e-mail: dev@tsklab.ru
                                                                              http://www.tsklab.ru
  CVS: $Id: CommingDBEditExtended.pas,v 1.11 2008/05/26 04:04:42 tsk Exp $
===================================================================================================}
unit CommingDBEditExtended;

interface

uses  Windows, Controls, Messages, Classes, SysUtils, DB, DBCtrls, Variants, Graphics,
      CommingEditExtended{, DesignIntf, CommingBinding}, CommingOther, CommingColor, Forms;

type

  TOnCheck = procedure( Sender: TObject; Value: Variant; var Valid: Boolean ) of object;

  { TcoDBDateEdit }

  TcoDBDateEdit = class( TcoCustomDateEdit )
  private     { Private declarations }
    FDataLink: TFieldDataLink;
    FCanvas: TControlCanvas;
    FOnCheck: TOnCheck;
    procedure DataChange(Sender: TObject);
    procedure EditingChange(Sender: TObject);
    function  GetDataField: string;
    function  GetDataSource: TDataSource;
    function  GetField: TField;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure SetReadOnly(Value: Boolean);
    procedure UpdateData(Sender: TObject);
    procedure AfterPopup(Sender: TObject; var Date: TDateTime; var Action: Boolean);
    procedure WMCut(var Message: TMessage); message WM_CUT;
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
    procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
//    function    ExecuteAction( Action: TBasicAction ): Boolean;
//    function UpdateAction(Action: TBasicAction): Boolean;
//    function  UseRightToLeftAlignment: Boolean;
  protected
    procedure AcceptValue(const Value: Variant); override;
    procedure ApplyDate(Value: TDateTime); override;
    function  GetReadOnly: Boolean; override;
    procedure Change; override;
    function  EditCanModify: Boolean; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Notification(AComponent: TComponent;Operation: TOperation); override;
    procedure Reset; override;
  public      { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   UpdateMask; override;
    function    Check: Boolean;
//    function    ExecuteAction(Action: TBasicAction): Boolean; override;
//    function    UpdateAction(Action: TBasicAction): Boolean; override;
//    function    UseRightToLeftAlignment: Boolean; override;
    property Field: TField read GetField;
  published   { Published declarations }
    property CalendarHints;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property AutoSelect;
    property BorderStyle default bsNone;
    property Height default 20;
    property BlanksChar;
    property ButtonHint;
    property CheckOnExit;
    property ClickKey;
    property Color;
//    property Ctl3D;
    property DefaultToday;
//    property DialogTitle;
    property DirectInput;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
//    property GlyphKind;
    { Ensure GlyphKind is declared before Glyph and ButtonWidth }
//    property Glyph;
//    property ButtonWidth;
    property HideSelection;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property ImeMode;
    property ImeName;
    property MaxLength;
//    property NumGlyphs;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupAlign;
    property PopupColor;
    property PopupMenu;
    property ShowHint;
//    property CalendarStyle;
    property TabOrder;
    property TabStop;
    property StartOfWeek;
    property Weekends;
    property WeekendColor;
//    property YearDigits;
    property Visible;
    property OnButtonClick;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnContextPopup;
    property    OnCheck: TOnCheck read FOnCheck write FOnCheck;
  end;

implementation

{ TcoDBDateEdit }

constructor TcoDBDateEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  inherited ReadOnly := True;
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnEditingChange := EditingChange;
  FDataLink.OnUpdateData := UpdateData;
  Self.OnAcceptDate := AfterPopup;
  AlwaysEnable := True;
  UpdateMask;
end;

destructor TcoDBDateEdit.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  FCanvas.Free;
  inherited Destroy;
end;

procedure TcoDBDateEdit.AfterPopup(Sender: TObject; var Date: TDateTime;
  var Action: Boolean);
begin
  Action := Action and (DataSource <> nil) and (DataSource.DataSet <> nil) and
    DataSource.DataSet.CanModify;
  if Action then Action := EditCanModify;
end;

procedure TcoDBDateEdit.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TcoDBDateEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if not ReadOnly and ((Key = VK_DELETE) or ((Key = VK_INSERT)
    and (ssShift in Shift))) then
    FDataLink.Edit;
end;

procedure TcoDBDateEdit.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if (Key in [#32..#255]) and (FDataLink.Field <> nil) and
    not (Key in ['0'..'9']) and (Key <> DateSeparator) then
  begin
    Beep;
    Key := #0;
  end;
  case Key of
    ^H, ^V, ^X, '0'..'9': FDataLink.Edit;
    #27:
      begin
        Reset;
        Key := #0;
      end;
  end;
end;

function TcoDBDateEdit.EditCanModify: Boolean;
begin
  Result := FDataLink.Edit;
end;

procedure TcoDBDateEdit.Reset;
begin
  FDataLink.Reset;
  SelectAll;
end;

procedure TcoDBDateEdit.Change;
begin
  if not Formatting then FDataLink.Modified;
  inherited Change;
end;

function TcoDBDateEdit.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TcoDBDateEdit.SetDataSource(Value: TDataSource);
begin
  if not (FDataLink.DataSourceFixed and (csLoading in ComponentState)) then
    FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

function TcoDBDateEdit.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TcoDBDateEdit.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

function TcoDBDateEdit.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TcoDBDateEdit.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
// TSK
  if not ( csDesigning in ComponentState ) then
    if Value then Color := clBtnFace else Color := clDimWindow;
  RecreateWnd;
end;

function TcoDBDateEdit.GetField: TField;
begin
  Result := FDataLink.Field;
end;

procedure TcoDBDateEdit.UpdateMask;
begin
  UpdateFormat;
  UpdatePopup;
  DataChange(nil);
end;

procedure TcoDBDateEdit.DataChange(Sender: TObject);
begin
  if FDataLink.Field <> nil then begin
    EditMask := GetDateMask;
    Self.Date := FDataLink.Field.AsDateTime;
  end
  else begin
    if csDesigning in ComponentState then begin
      EditMask := '';
      EditText := Name;
    end
    else begin
      EditMask := GetDateMask;
      if DefaultToday
        then Date := Trunc( Now ) // SysUtils.Date
        else Date := 0; //TSK
    end;
  end;
end;

procedure TcoDBDateEdit.EditingChange(Sender: TObject);
begin
  inherited ReadOnly := not FDataLink.Editing;
  if FDataLink.Editing and DefaultToday and (FDataLink.Field <> nil) and
    (FDataLink.Field.AsDateTime = 0) then
    FDataLink.Field.AsDateTime := Trunc( Now );     // TSK
end;

procedure TcoDBDateEdit.UpdateData(Sender: TObject);
var
  D: TDateTime;
begin
  ValidateEdit;
  D := Self.Date;
  if D <> 0 then
    FDataLink.Field.AsDateTime := D // + Frac(FDataLink.Field.AsDateTime)
  else FDataLink.Field.Clear;
end;

procedure TcoDBDateEdit.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

procedure TcoDBDateEdit.WMPaint(var Message: TWMPaint);
var
  S: string;
begin
  if (csPaintCopy in ControlState) and (FDataLink.Field <> nil) then begin
    if FDataLink.Field.IsNull then begin
      S := GetDateFormat;
//      S := ReplaceStr(ReplaceStr(ReplaceStr(ReplaceStr(S, '/', DateSeparator),
//        'Y', ' '), 'M', ' '), 'D', ' ');
    end
    else
      S := FormatDateTime(GetDateFormat, FDataLink.Field.AsDateTime);
  end else S := EditText;
//  if not PaintComboEdit(Self, S, Alignment, True, FCanvas, Message) then
    inherited;
end;

procedure TcoDBDateEdit.AcceptValue(const Value: Variant);
begin
  if VarIsNull(Value) or VarIsEmpty(Value)
    then FDataLink.Field.Clear
    else FDataLink.Field.AsDateTime := VarToDateTime( Value );// + Frac(FDataLink.Field.AsDateTime);
  DoChange;
end;

procedure TcoDBDateEdit.ApplyDate(Value: TDateTime);
begin
  FDataLink.Edit;
  inherited ApplyDate(Value);
end;

procedure TcoDBDateEdit.WMPaste(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

procedure TcoDBDateEdit.WMCut(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

procedure TcoDBDateEdit.CMEnter(var Message: TCMEnter);
begin
  inherited;
end;

procedure TcoDBDateEdit.CMExit(var Message: TCMExit);
begin
  try
    if not (csDesigning in ComponentState) and CheckOnExit then
      CheckValidDate;
    FDataLink.UpdateRecord;
  except
    SelectAll;
    if CanFocus then SetFocus;
    raise;
  end;
  CheckCursor;
  DoExit;
end;
//==================================================================================================
function TcoDBDateEdit.Check: Boolean;
begin
  if Assigned( FOnCheck ) and ( Field <> nil )
    then FOnCheck( Self, Field.Value, Result )
    else Result := True;
  if not Result then SetFocusShowHint( Self );
end;
//==================================================================================================
end.
