{===================================================================================================

  Comming Library

     .

====================================================================================================

  :

  04.01.2010 [*]  ""    ""    .
  24.03.2004 [*] .
  10.12.2003 [*]  uses (Icarus).
  04.06.2003 [*]   .
  25.05.2003 [+]  TActionList.
  21.05.2003 [*]  .
             [*]  resourcestring.
  06.05.2003 [*]   .
  04.05.2003 [*]   .
  29.11.2002 [*]    CommingDesign  CommingRuntime
  27.08.2002 [*]  TcoToolbarButton97  TcoButton.
  16.07.2002 [*]     .
  09.07.2002 [+]  .
             [+]   .
  08.07.2002 [*]   .
             [-]   .
             [+]    ,     .
  10.02.2002 [*]     .
  01.11.2001 [*]  EAbort (   )  EUser.
  08.10.2001 [*]  raise  Application.ShowException;
  12.06.2001 [!]   Delphi6.
  11.03.2001 [+]  .  .
  09.09.1998 [!]  Comming Library.

====================================================================================================
  Copyright 19982010 TSK Developer Laboratory, Moscow.                     Konstantin Tsvetkov
                                                                              e-mail: dev@tsklab.ru
                                                                              http://www.tsklab.ru
  CVS: $Id: CommingError.pas,v 1.22 2010/01/25 06:30:29 tsk Exp $
===================================================================================================}
unit CommingError;

interface

uses  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
      Dialogs, ExtCtrls, ImgList, StdCtrls, TypInfo, Math, ActnList,
      // Comming:
      CommingSystem, CommingVersionInfo, CommingLog, CommingMAPI, CommingButton;

type

  EUser = class( Exception );

  TcoOption = ( eoSendMail, eoWriteLog, eoShowFull, eoTerminate );
  TcoOptions = set of TcoOption;

  TErrorFR = class(TForm)
    btContinue: TcoButton;
    laMessage: TLabel;
    btMail: TcoButton;
    btSwitch: TcoButton;
    imError: TImageList;
    imIcon: TImage;
    alError: TActionList;
    acSend: TAction;
    acShowFull: TAction;
    laFull: TLabel;
    acShowShort: TAction;
    acContinue: TAction;
    acExit: TAction;
    btExit: TcoButton;
    procedure FormCreate(Sender: TObject);
    procedure SendError( Sender: TObject );
    procedure ShowFull( Sender: TObject );
    procedure ShowShort( Sender: TObject );
    procedure ContinueExecute( Sender: TObject );
    procedure ExitExecute( Sender: TObject );
  private   { Private declarations }
    FOption: TcoOptions;
    FExceptionEvent: TExceptionEvent;
    FTo:      String;
    FSubj:    String;
    FFrom:    String;
    procedure   AdjustFormSize;
    procedure   SetOption( const Value: TcoOptions );
    property    Option: TcoOptions read FOption write SetOption;
  end;

  TcoError = class( TComponent )
  private
    FForm: TErrorFR;
    FEMail: String;
    FOption: TcoOptions;
    procedure   SetOption( const Value: TcoOptions );
  public
    constructor Create( AOwner: TComponent ); override;
    procedure   DoException( Sender: TObject; Error: Exception );
  published
    property    Option: TcoOptions read FOption write SetOption default [ eoSendMail, eoWriteLog, eoShowFull ];
    property    EMail: String read FEMail write FEMail;
  end;

resourcestring
    rsObjectName  = '%s: %s(%s)'#13#10;
    rsNotDefined  = 'Not defined';
    rsNotName     = 'Not name';
    rsSubject     = '%s %s:  (%s)  !';
    rsFull        = ':'#13#10'%s: %s';

implementation

{$R *.dfm}
//==================================================================================================
function GetUnitName( xClass: TClass ): String;
begin
  try
    Result := GetTypeData( PTypeInfo( xClass.ClassInfo ))^.UnitName;
  except
    Result := rsNotDefined;
  end;
end;
//==================================================================================================
function GetSubordination( xObject: TObject ): String ;
  var xOwner: TComponent;
  function EmptyName( Name: String ): String;
  begin
    if Length( Name ) = 0 then Result := rsNotName else Result := Name;
  end;
begin
  if ( xObject is TControl ) then
    with ( xObject as TControl ) do begin
      if Parent = nil then xOwner := Owner else xOwner  := Parent;
      Result := Format( rsObjectName, [ EmptyName( Name ), ClassName, GetUnitName( ClassType ) ]);
    end
  else if ( xObject is TComponent ) then
    with ( xObject as TComponent ) do begin
      xOwner  := Owner;
      Result := Format( rsObjectName, [ EmptyName( Name ), ClassName, GetUnitName( ClassType )]);
    end
  else if ( xObject is TObject ) then
    with ( xObject as TObject ) do begin
      Result := Format( rsObjectName, [ rsNotName, ClassName, GetUnitName( ClassType )]);
      xOwner := nil;
    end
  else Exit;
  Result := Result + GetSubordination( xOwner )
end;
//==================================================================================================
{ TcoError }
//==================================================================================================
constructor TcoError.Create( AOwner: TComponent );
begin
  inherited Create(AOwner);
  Application.OnException := DoException;
  Option := [ eoSendMail, eoWriteLog, eoShowFull ]
end;
//==================================================================================================
procedure TcoError.DoException( Sender: TObject; Error: Exception );
begin
  Screen.Cursor := crDefault;
  Application.OnException := nil;
  if FForm = nil then begin
    Application.CreateForm( TErrorFR, FForm );
    FForm.Caption         := Application.Title + ' []';
    FForm.FExceptionEvent := DoException;
    FForm.Option          := Option;
  end;
  FForm.FSubj             := Format( rsSubject,
                                    [ Application.Title, GetVersion( '', False ), Error.ClassName ]);
  FForm.laMessage.Caption := Error.Message + '.';
  FForm.laFull.Caption    := Format( rsFull, [ GetSubordination( Sender ), Application.Hint ]);
  FForm.FFrom             := GetRegisteredOwner + ', ' + GetRegisteredOrganization + '.';
  FForm.FTo               := EMail;
  if eoWriteLog in Option
    then WriteLog( FForm.FSubj + #13#10#13#10
                   + FForm.laMessage.Caption + #13#10#13#10
                   + FForm.laFull.Caption + #13#10#13#10
                   + FForm.FFrom );
  MessageBeep( MB_ICONHAND );
  FForm.AdjustFormSize;
  FForm.ShowModal;
end;
//==================================================================================================
procedure TcoError.SetOption( const Value: TcoOptions );
begin
  FOption := Value;
  if FForm <> nil then FForm.Option := Value
end;
//==================================================================================================
{ TErrorFR }
//==================================================================================================
procedure TErrorFR.FormCreate( Sender: TObject );
begin
  imIcon.Picture.Icon.Handle  := LoadIcon(0, IDI_ERROR );
  AdjustFormSize;
end;
//==================================================================================================
procedure TErrorFR.SendError( Sender: TObject );
begin
  SendMail( FTo, FSubj, laMessage.Caption + #13#10#13#10
                        + laFull.Caption + #13#10#13#10
                        + FFrom )
end;
//==================================================================================================
procedure TErrorFR.ShowFull( Sender: TObject );
begin
  laFull.Visible  := True;
  btSwitch.Action := acShowShort;
  AdjustFormSize;
end;
//==================================================================================================
procedure TErrorFR.ShowShort( Sender: TObject );
begin
  laFull.Visible  := False;
  btSwitch.Action := acShowFull;
  AdjustFormSize;
end;
//==================================================================================================
procedure TErrorFR.ContinueExecute(Sender: TObject);
begin
  Close;
  Application.OnException := FExceptionEvent;
end;
//==================================================================================================
procedure TErrorFR.ExitExecute(Sender: TObject);
begin
  Application.Terminate;
end;
//==================================================================================================
procedure TErrorFR.SetOption( const Value: TcoOptions );
begin
  FOption := Value;
  // SendMait
  acSend.Enabled    := ( eoSendMail in Value ) and ( Length( FTo ) > 0 );
  // ShowFull
  btSwitch.Enabled  := ( eoShowFull in Value );
  laFull.Visible    := btSwitch.Enabled;
  // Terminate
  if eoTerminate in Option then begin
    btContinue.Visible := False;
    btExit.Left        := ClientWidth - btExit.Width - 8;
  end;
end;
//==================================================================================================
procedure TErrorFR.AdjustFormSize;
begin
  acSend.Enabled    := ( eoSendMail in FOption ) and ( Length( FTo ) > 0 );
  laMessage.Width := ClientWidth - laMessage.Left - 16;
  laFull.Width    := laMessage.Width;
  laFull.Top      := laMessage.Top + laMessage.Height + 4;
  if laFull.Visible
    then Height   := Max( laFull.Top + laFull.Height, imIcon.Top + imIcon.Height )
                     + ( Height - btExit.Top ) + 8
    else Height   := Max( laFull.Top - 4, imIcon.Top + imIcon.Height )
                     + ( Height - btExit.Top ) + 8;
//  btExit.Left     := ClientWidth - btExit.Width - 8;
end;
//==================================================================================================
end.
