{===================================================================================================

  Comming Library.

    TcoHint.

====================================================================================================

  :

  29.01.2005  [*] .
  22.05.2003  [*]  .
              [+]  .
              [*] .
  21.05.2003  [+] .
  16.02.2003  [+] IsFlatHint:    TcoHint;
  29.11.2002  [+]  Hint.
  01.12.2001  [-]   FlatStyle.
  30.11.2001  [+]   FlatStyle.
  09.09.1998  [!]  Comming Library.

====================================================================================================
  Copyright 19982005 TSK Developer Laboratory, Moscow.                     Konstantin Tsvetkov
                                                                              e-mail: dev@tsklab.ru
                                                                              http://www.tsklab.ru
  CVS: $Id: CommingHint.pas,v 1.14 2007/08/17 07:53:17 tsk Exp $
===================================================================================================}
unit CommingHint;

interface

uses  Classes, Windows, Graphics, Messages, Controls, Forms, SysUtils,
      // Comming:
      CommingColor;

type

  TArrowPos = ( NE, NW, SE, SW );

  TcoHint = class(TComponent)
  private
    FHintFont: TFont;
    FBackgroundColor: TColor;
    FBorderColor: TColor;
    FArrowBackgroundColor: TColor;
    FArrowColor: TColor;
    FWindowHandle: HWND;
    FHintWidth: Integer;
    FAlphaBlend: Boolean;
    FSystemColor: Boolean;
    procedure   GetHintInfo( var HintStr: string; var CanShow: Boolean; var HintInfo: THintInfo );
    procedure   SetSystemColor( const Value: Boolean );
  public
    constructor Create (AOwner: TComponent); override;
    destructor  Destroy; override;
  published
    property    AlphaBlend:  Boolean read FAlphaBlend  write FAlphaBlend    default True;
    property    SystemColor: Boolean read FSystemColor write SetSystemColor default False;
    property    HintWidth:   Integer read FHintWidth   write FHintWidth     default 500;
  end;

  TFlatHintWindow = class( THintWindow )
  private
    FArrowPos: TArrowPos;
    FArrowPoint: TPoint;
  protected
    procedure   Paint; override;
    procedure   CreateParams( var Params: TCreateParams ); override;
    procedure   CreateWindowHandle( const Params: TCreateParams ); override;
  public
    procedure   ActivateHint( HintRect: TRect; const AHint: string ); override;
  end;

implementation

  var HintControl: TControl;
      FHint: TcoHint;

//==================================================================================================
{ TcoHint }
//==================================================================================================
constructor TcoHint.Create( AOwner: TComponent );
begin
  inherited Create(AOwner);

  FHint := Self;
  FAlphaBlend    := True;

  if not (csDesigning in ComponentState) then begin
    HintWindowClass := TFlatHintWindow;

    with Application do begin
      ShowHint    := not ShowHint;
      ShowHint    := not ShowHint;
      OnShowHint  := GetHintInfo;

      HintShortPause  := 25;
      HintPause       := 500;
      HintHidePause   := 10000;
    end;
  end;
  SystemColor         := False;
  HintWidth           := 500;
  FHintFont           := TFont.Create;      //Screen.HintFont
  FHintFont.Name      := 'Tahoma';

end;
//==================================================================================================
destructor TcoHint.Destroy;
begin
  FHintFont.Free;
  if not ( csDesigning in ComponentState ) then DeallocateHWnd( FWindowHandle );
  inherited Destroy;
end;
//==================================================================================================
procedure TcoHint.GetHintInfo ( var HintStr: string; var CanShow: Boolean; var HintInfo: THintInfo );
begin
  HintControl := HintInfo.HintControl;
end;
//==================================================================================================
procedure TcoHint.SetSystemColor(const Value: Boolean);
begin
  FSystemColor := Value;
  if FSystemColor then begin
    FBackgroundColor      := clInfoBk;
    FBorderColor          := clInfoText;
    FArrowBackgroundColor := clInfoText;
    FArrowColor           := clInfoBk;
  end else begin
    FBackgroundColor      := clDimWindow;
    FBorderColor          := clWindowFrame;
    FArrowBackgroundColor := clWindowFrame;//clInactiveCaption;
    FArrowColor           := clDimWindow;
  end;
end;
//==================================================================================================
{ TcoHintWindow }
//==================================================================================================
procedure TFlatHintWindow.CreateParams( var Params: TCreateParams );
begin
  inherited CreateParams( Params );
  Params.Style := Params.Style - WS_BORDER;
  if FHint.AlphaBlend and Assigned( SetLayeredWindowAttributes )
    then Params.ExStyle := Params.ExStyle or WS_EX_LAYERED;
end;
//==================================================================================================
procedure TFlatHintWindow.CreateWindowHandle( const Params: TCreateParams );
begin
  inherited;
  if FHint.AlphaBlend and Assigned( SetLayeredWindowAttributes )
    then SetLayeredWindowAttributes( Handle, 0, 220, LWA_ALPHA );
end;
//==================================================================================================
procedure TFlatHintWindow.Paint;
var
  ArrowRect, TextRect: TRect;
begin

  // Set the Rect's
  case FArrowPos of
    NW, SW:
      begin
        ArrowRect := Rect(ClientRect.Left + 1, ClientRect.Top + 1, ClientRect.Left + 15, ClientRect.Bottom - 1);
        TextRect  := Rect(ClientRect.Left + 15, ClientRect.Top + 1, ClientRect.Right - 1, ClientRect.Bottom - 1);
      end;
    NE, SE:
      begin
        ArrowRect := Rect(ClientRect.Right - 15, ClientRect.Top + 1, ClientRect.Right - 1, ClientRect.Bottom - 1);
        TextRect  := Rect(ClientRect.Left + 1, ClientRect.Top + 1, ClientRect.Right - 15, ClientRect.Bottom - 1);
      end;
  end;

  // DrawBackground
  Canvas.Brush.Color := FHint.FArrowBackgroundColor;
  Canvas.FillRect( ArrowRect );
  canvas.brush.color := FHint.FBackgroundColor;
  canvas.FillRect( TextRect );

  // DrawBorder
  canvas.Brush.Color := FHint.FBorderColor;
  canvas.FrameRect(ClientRect);

  // DrawArrow
  case FArrowPos of
    NW: FArrowPoint := Point(ArrowRect.Left + 2, ArrowRect.Top + 2);
    NE: FArrowPoint := Point(ArrowRect.Right - 3, ArrowRect.Top + 2);
    SW: FArrowPoint := Point(ArrowRect.Left + 2, ArrowRect.Bottom - 3);
    SE: FArrowPoint := Point(ArrowRect.Right - 3, ArrowRect.Bottom - 3);
  end;
  canvas.Pen.Color := FHint.FArrowColor;
  case FArrowPos of
    NW: canvas.Polyline([Point(FArrowPoint.x,     FArrowPoint.y),     Point(FArrowPoint.x, FArrowPoint.y + 6),
                         Point(FArrowPoint.x + 1, FArrowPoint.y + 6), Point(FArrowPoint.x + 1, FArrowPoint.y),
                         Point(FArrowPoint.x + 6, FArrowPoint.y),     Point(FArrowPoint.x + 6, FArrowPoint.y + 1),
                         Point(FArrowPoint.x + 2, FArrowPoint.y + 1), Point(FArrowPoint.x + 2, FArrowPoint.y + 4),
                         Point(FArrowPoint.x + 5, FArrowPoint.y + 7), Point(FArrowPoint.x + 6, FArrowPoint.y + 7),
                         Point(FArrowPoint.x + 3, FArrowPoint.y + 4), Point(FArrowPoint.x + 3, FArrowPoint.y + 3),
                         Point(FArrowPoint.x + 6, FArrowPoint.y + 6), Point(FArrowPoint.x + 7, FArrowPoint.y + 6),
                         Point(FArrowPoint.x + 3, FArrowPoint.y + 2), Point(FArrowPoint.x + 4, FArrowPoint.y + 2),
                         Point(FArrowPoint.x + 7, FArrowPoint.y + 5), Point(FArrowPoint.x + 7, FArrowPoint.y + 6)]);
    NE: canvas.Polyline([Point(FArrowPoint.x,     FArrowPoint.y),     Point(FArrowPoint.x, FArrowPoint.y + 6),
                         Point(FArrowPoint.x - 1, FArrowPoint.y + 6), Point(FArrowPoint.x - 1, FArrowPoint.y),
                         Point(FArrowPoint.x - 6, FArrowPoint.y),     Point(FArrowPoint.x - 6, FArrowPoint.y + 1),
                         Point(FArrowPoint.x - 2, FArrowPoint.y + 1), Point(FArrowPoint.x - 2, FArrowPoint.y + 4),
                         Point(FArrowPoint.x - 5, FArrowPoint.y + 7), Point(FArrowPoint.x - 6, FArrowPoint.y + 7),
                         Point(FArrowPoint.x - 3, FArrowPoint.y + 4), Point(FArrowPoint.x - 3, FArrowPoint.y + 3),
                         Point(FArrowPoint.x - 6, FArrowPoint.y + 6), Point(FArrowPoint.x - 7, FArrowPoint.y + 6),
                         Point(FArrowPoint.x - 3, FArrowPoint.y + 2), Point(FArrowPoint.x - 4, FArrowPoint.y + 2),
                         Point(FArrowPoint.x - 7, FArrowPoint.y + 5), Point(FArrowPoint.x - 7, FArrowPoint.y + 6)]);
    SW: canvas.Polyline([Point(FArrowPoint.x,     FArrowPoint.y),     Point(FArrowPoint.x, FArrowPoint.y - 6),
                         Point(FArrowPoint.x + 1, FArrowPoint.y - 6), Point(FArrowPoint.x + 1, FArrowPoint.y),
                         Point(FArrowPoint.x + 6, FArrowPoint.y),     Point(FArrowPoint.x + 6, FArrowPoint.y - 1),
                         Point(FArrowPoint.x + 2, FArrowPoint.y - 1), Point(FArrowPoint.x + 2, FArrowPoint.y - 4),
                         Point(FArrowPoint.x + 5, FArrowPoint.y - 7), Point(FArrowPoint.x + 6, FArrowPoint.y - 7),
                         Point(FArrowPoint.x + 3, FArrowPoint.y - 4), Point(FArrowPoint.x + 3, FArrowPoint.y - 3),
                         Point(FArrowPoint.x + 6, FArrowPoint.y - 6), Point(FArrowPoint.x + 7, FArrowPoint.y - 6),
                         Point(FArrowPoint.x + 3, FArrowPoint.y - 2), Point(FArrowPoint.x + 4, FArrowPoint.y - 2),
                         Point(FArrowPoint.x + 7, FArrowPoint.y - 5), Point(FArrowPoint.x + 7, FArrowPoint.y - 6)]);
    SE: canvas.Polyline([Point(FArrowPoint.x,     FArrowPoint.y),     Point(FArrowPoint.x, FArrowPoint.y - 6),
                         Point(FArrowPoint.x - 1, FArrowPoint.y - 6), Point(FArrowPoint.x - 1, FArrowPoint.y),
                         Point(FArrowPoint.x - 6, FArrowPoint.y),     Point(FArrowPoint.x - 6, FArrowPoint.y - 1),
                         Point(FArrowPoint.x - 2, FArrowPoint.y - 1), Point(FArrowPoint.x - 2, FArrowPoint.y - 4),
                         Point(FArrowPoint.x - 5, FArrowPoint.y - 7), Point(FArrowPoint.x - 6, FArrowPoint.y - 7),
                         Point(FArrowPoint.x - 3, FArrowPoint.y - 4), Point(FArrowPoint.x - 3, FArrowPoint.y - 3),
                         Point(FArrowPoint.x - 6, FArrowPoint.y - 6), Point(FArrowPoint.x - 7, FArrowPoint.y - 6),
                         Point(FArrowPoint.x - 3, FArrowPoint.y - 2), Point(FArrowPoint.x - 4, FArrowPoint.y - 2),
                         Point(FArrowPoint.x - 7, FArrowPoint.y - 5), Point(FArrowPoint.x - 7, FArrowPoint.y - 6)]);
  end;

  // DrawHintText
  Canvas.Brush.Style := bsClear;
  InflateRect( TextRect, -3, -1 );
  DrawText( Canvas.Handle, PChar(Caption), Length(Caption), TextRect, DT_WORDBREAK or DT_NOPREFIX);
end;

procedure TFlatHintWindow.ActivateHint (HintRect: TRect; const AHint: string);
  var curWidth: Byte;
      Pnt: TPoint;
      HintHeight, HintWidth: Integer;
      NordWest, NordEast, SouthWest, SouthEast: TRect;
begin
  Caption := AHint;

  Canvas.Font.Assign(FHint.FHintFont);

  // Calculate width and height
  HintRect.Right := HintRect.Left + FHint.FHintWidth - 22;
  DrawText(Canvas.Handle, @AHint[1], Length(AHint), HintRect, DT_CALCRECT or DT_WORDBREAK or DT_NOPREFIX);
  Inc(HintRect.Right, 22);
  Inc(HintRect.Bottom, 6);

  // Divide the screen in 4 pices
  NordWest :=  Rect(0, 0, Screen.Width div 2, Screen.Height div 2);
  NordEast :=  Rect(Screen.Width div 2, 0, Screen.Width, Screen.Height div 2);
  SouthWest := Rect(0, Screen.Height div 2, Screen.Width div 2, Screen.Height);
  SouthEast := Rect(Screen.Width div 2, Screen.Height div 2, Screen.Width, Screen.Height);

  GetCursorPos(Pnt);

  if PtInRect(NordWest, Pnt) then
    FArrowPos := NW
  else
    if PtInRect(NordEast, Pnt) then
      FArrowPos := NE
    else
      if PtInRect(SouthWest, Pnt) then
        FArrowPos := SW
      else
        FArrowPos := SE;

  // Calculate the position of the hint
  if FArrowPos = NW then
    curWidth := 6//12
  else
    curWidth := 3;//5;

  HintHeight := HintRect.Bottom - HintRect.Top;
  HintWidth  := HintRect.Right - HintRect.Left;

  case FArrowPos of
    NW: HintRect := Rect(Pnt.x + curWidth, Pnt.y + curWidth, Pnt.x + HintWidth + curWidth, Pnt.y + HintHeight + curWidth);
    NE: HintRect := Rect(Pnt.x - HintWidth - curWidth, Pnt.y + curWidth, Pnt.x - curWidth, Pnt.y + HintHeight + curWidth);
    SW: HintRect := Rect(Pnt.x + curWidth, Pnt.y - HintHeight - curWidth, Pnt.x + HintWidth + curWidth, Pnt.y - curWidth);
    SE: HintRect := Rect(Pnt.x - HintWidth - curWidth, Pnt.y - HintHeight - curWidth, Pnt.x - curWidth, Pnt.y - curWidth);
  end;

  BoundsRect := HintRect;

  Pnt := ClientToScreen(Point(0, 0));

  SetWindowPos(Handle, HWND_TOPMOST, Pnt.X, Pnt.Y, 0, 0, SWP_SHOWWINDOW or SWP_NOACTIVATE or SWP_NOSIZE);
end;
//==================================================================================================
end.
