{ ==================================================================================================

  Comming Library

      Toolbar97.

===================================================================================================}
unit Comming97Utils;

interface
Uses
   Windows, Classes, Graphics;

Type
  TFontListOption = (foAnsiOnly, foTrueTypeOnly, foFixedPitchOnly,
    foNoOEMFonts, foOEMFontsOnly, foScalableOnly);
  TFontListOptions = set of TFontListOption;

function Sign(const Value : Extended): ShortInt;
function MakeModuleBitmap(Module: THandle; ResID: PChar): TBitmap;
//function CreateBitmap(ResName: PChar): TBitmap;
function PointInRect(P: TPoint; const R: TRect): Boolean;
procedure Delay( MSecs: DWORD; AllowAccess : Boolean );
Procedure BuildFontFaceList(aFontList : TStrings;
                            aDC: HDC;
                            aOption : TFontListOptions);

implementation
Uses
  SysUtils, Forms;

Var
  tmpFontsList       : TStringList;
  tmpFontOptions     : TFontListOptions;
  tmpDC              : hDC;

function Sign(const Value : Extended): ShortInt;
Begin
  Result := 1;
  If Value < 0 Then
    Result := -1
End;

function MakeModuleBitmap(Module: THandle; ResID: PChar): TBitmap;
begin
  Result := TBitmap.Create;
  Result.Handle := LoadBitmap(Module, ResID);
  if Result.Handle = 0 then begin
    Result.Free;
    Result := nil;
  end;
end;

{function CreateBitmap(ResName: PChar): TBitmap;
begin
  Result := MakeModuleBitmap(HInstance, ResName);
  if Result = nil then Exception.Create('ResourceNotFound');
end;}

function PointInRect(P: TPoint; const R: TRect): Boolean;
begin
  with R do
    Result := (Left <= P.X) and (Top <= P.Y) and
      (Right >= P.X) and (Bottom >= P.Y);
end;

procedure Delay( MSecs: DWORD; AllowAccess : Boolean );
  var FirstTickCount: DWORD;
begin
  FirstTickCount := GetTickCount;
  repeat
    if AllowAccess then Application.ProcessMessages;
  until (( GetTickCount - FirstTickCount ) >= MSecs );
end;


function EnumFontFacesProc(var LogFont: TLogFont; var TextMetric: TTextMetric;
                    FontType: Integer; Data: Pointer): Integer; export; stdcall;
Var
  Valid : Boolean;
begin
  if (tmpFontsList.IndexOf(StrPas(LogFont.lfFaceName)) < 0) then Begin
    Valid := True;
    if foAnsiOnly in tmpFontOptions then
      Valid := Valid and (LogFont.lfCharSet = ANSI_CHARSET);
    if foTrueTypeOnly in tmpFontOptions then
      Valid := Valid and (FontType and TRUETYPE_FONTTYPE = TRUETYPE_FONTTYPE);
    if foFixedPitchOnly in tmpFontOptions then
      Valid := Valid and (LogFont.lfPitchAndFamily and FIXED_PITCH = FIXED_PITCH);
    if foOEMFontsOnly in tmpFontOptions then
      Valid := Valid and (LogFont.lfCharSet = OEM_CHARSET);
    if foNoOEMFonts in tmpFontOptions then
      Valid := Valid and (LogFont.lfCharSet <> OEM_CHARSET);
    if foScalableOnly in tmpFontOptions then
      Valid := Valid and (FontType and RASTER_FONTTYPE = 0);
    If Valid Then
      tmpFontsList.AddObject(StrPas(LogFont.lfFaceName), TObject(FontType));
  End;
  Result := 1;
end;


function EnumAllFontsProc(var LogFont: TLogFont; var TextMetric: TTextMetric;
                    FontType: Integer; Data: Pointer): Integer; export; stdcall;
begin
  EnumFonts(tmpDC, LogFont.lfFaceName, @EnumFontFacesProc, Data);
  Result := 1;
end;



Procedure BuildFontFaceList(aFontList : TStrings;
                            aDC: HDC;
                            aOption : TFontListOptions);
Begin
  aFontList.Clear;
  tmpFontsList := TStringList.Create;
  tmpFontsList.Sorted := True;
  tmpDC := aDC;
  tmpFontOptions := aOption;
  Try
    EnumFonts(tmpDC, nil, @EnumAllFontsProc, Pointer(tmpFontsList));
    aFontList.Assign(tmpFontsList);
  Finally
    tmpFontsList.Free;
  End;

End;

end.
