{==================================================================================================

  Comming Library

        Pervasive.SQL.

====================================================================================================

  :

  20.09.2001    Delphi6.
  29.05.2001   TAB.
  21.08.2000         Pervasicve.SQL.
                        Pervasicve.SQL.
                     Pervasicve.SQL.
  09.09.1998 TSK   Comming Library.

==================================================================================================
  Copyright 1999-2001 TSK Developer Laboratory, Moscow.                 Konstantin Tsvetkov
                                                                            e-mail: tsklab@mail.ru
===================================================================================================}
unit CommingBtrieve;

interface

  uses SysUtils, Forms, BseMain, BtrMain, CommingDialog, DB, CommingRegistry, Windows, Variants;

  function  PSQLExecute( Query: TtbQuery; SQLText: String ): Boolean; overload;
  function  PSQLExecute( Query: TtbQuery; SQLText: String; const Args: array of const ): Boolean; overload;
  function  PSQLExecute( SQLDataBase, SQLText: String ): Boolean; overload;
  function  PSQLExecute( SQLDataBase, SQLText: String; const Args: array of const ): Boolean; overload;
  function  PSQLCalculate( Query: TtbQuery; SQLText: String ): Variant; overload;
  function  PSQLCalculate( Query: TtbQuery; SQLText: String; const Args: array of const ): Variant; overload;
  function  PSQLCalculate( SQLDataBase, SQLText: String ): Variant; overload;
  function  PSQLCalculate( SQLDataBase, SQLText: String; const Args: array of const ): Variant; overload;
  function  PSQLListing( Query: TtbQuery; SQLText: String ): Variant; overload;
  function  PSQLListing( Query: TtbQuery; SQLText: String; const Args: array of const ): Variant; overload;
  function  PSQLBeginTransaction( Query: TtbQuery ): Boolean;
  procedure  PSQLCommit( Query: TtbQuery );
  procedure  PSQLRollback( Query: TtbQuery );

  procedure BtrieveException( Error: Exception; Operation: String  = '' );

  procedure  SetSettingBeforeQSL;
  procedure  RestoreSettingAfterQSL;

  function  TitanReadPath( Alias: String ): String;
  function  TitanReadDescription( Alias: String ): String;
  procedure TitanWritePath( Alias, Path: String );
  procedure TitanWriteDescription( Alias, Description: String );

  const TitanNotPath = '<not>';

  var sDatSeparat: Char;
      sShortDatFormat: string;
      sDecimSeparat: Char;

implementation
//==================================================================================================
function  PSQLExecute( Query: TtbQuery; SQLText: String ): Boolean; overload;
begin
  with Query do begin
    SQL.Text := SQLText;
    ExecSQL;
  end;
  Result := True;
end;
//==================================================================================================
function  PSQLExecute( Query: TtbQuery; SQLText: String; const Args: array of const ): Boolean; overload;
begin
  Result := PSQLExecute( Query, Format( SQLText, Args ));
end;
//==================================================================================================
function  PSQLExecute( SQLDataBase, SQLText: String ): Boolean; overload;
  var Query: TtbQuery;
begin
  Query := TtbQuery.Create(nil);
  try
    with Query do begin
      DatabaseName := SQLDataBase;
      SQL.Text := SQLText;
      ExecSQL;
    end;
    Result := True;
  finally
    Query.Free;
  end;
end;
//==================================================================================================
function PSQLExecute( SQLDataBase, SQLText: String; const Args: array of const ): Boolean; overload;
begin
  Result := PSQLExecute( SQLDataBase, Format( SQLText, Args ));
end;
//==================================================================================================
function  PSQLCalculate( Query: TtbQuery; SQLText: String ): Variant; overload;
  var i: Integer;
begin
  with Query do begin
    Close;
    SQL.Text := SQLText;
//    SQL.Add( SQLText );
    try
      Open;
      if eof then Result := 0
      else begin
        if Fields.Count > 1 then begin
          Result := VarArrayCreate( [ 0, Fields.Count - 1 ], varVariant );
          for i := 0 to Fields.Count - 1 do begin
            if Fields[i] is TStringField then ( Fields[i] as TStringField ).Transliterate := False;
            Result[i] := Fields[i].AsVariant;
          end;
        end else begin
          if Fields[0] is TStringField then ( Fields[0] as TStringField ).Transliterate := False;
          Result := Fields[0].AsVariant;
        end
      end;
    except on E: Exception do
      if E.Message = 'DBIERR_EOF'
        then Result := 0
        else raise E
    end;
    Close;
  end;
end;
//==================================================================================================
function PSQLCalculate( Query: TtbQuery; SQLText: String; const Args: array of const ): Variant; overload;
begin
  Result := PSQLCalculate( Query, Format( SQLText, Args ));
end;
//==================================================================================================
function  PSQLCalculate( SQLDataBase, SQLText: String ): Variant; overload;
  var Query: TtbQuery;
      i: Integer;
begin
  Query := TtbQuery.Create(nil);
  try
    with Query do begin
      DatabaseName := SQLDataBase;
      SQL.Text := SQLText;
      try
      Open;
      if eof then Result := 0
      else begin
        if Fields.Count > 1 then begin
          Result := VarArrayCreate( [ 0, Fields.Count - 1 ], varVariant );
          for i := 0 to Fields.Count - 1 do begin
            if Fields[i] is TStringField then ( Fields[i] as TStringField ).Transliterate := False;
            Result[i] := Fields[i].AsVariant;
          end;
        end else begin
          if Fields[0] is TStringField then ( Fields[0] as TStringField ).Transliterate := False;
          Result := Fields[0].AsVariant;
        end
      end;
      except on E: Exception do
        if E.Message = 'DBIERR_EOF'
          then Result := 0
          else raise
      end;
      Close;
    end;
  finally
    Query.Free;
  end;
end;
//==================================================================================================
function PSQLCalculate( SQLDataBase, SQLText: String; const Args: array of const ): Variant; overload;
begin
  Result := PSQLCalculate( SQLDataBase, Format( SQLText, Args ));
end;
//==================================================================================================
function PSQLListing( Query: TtbQuery; SQLText: String ): Variant; overload;
  var i: Integer;
begin
  Result := 0;
  with Query do begin
    Close;
    SQL.Text := SQLText;
    try
      Open;
      if RecordCount < 1 then begin
        Result := -1;
        Close;
        Exit;
      end;
      Result := VarArrayCreate([ 1, RecordCount ], varVariant );
      i := 1;
      if Fields[0] is TStringField then ( Fields[0] as TStringField ).Transliterate := False;
      while not eof do begin
        Result[i] := Fields[0].AsVariant;
        Inc( i );
        Next;
      end;
    except on E: Exception do
      if not ( E.Message = 'DBIERR_EOF' ) then raise;
    end;
    Close;
  end;
end;
//==================================================================================================
function PSQLListing( Query: TtbQuery; SQLText: String; const Args: array of const ): Variant; overload;
begin
  Result := PSQLListing( Query, Format( SQLText, Args ));
end;
//==================================================================================================
function  PSQLBeginTransaction( Query: TtbQuery): Boolean;
begin
  with Query do begin
    SQL.Text := 'START TRANSACTION';
    ExecSQL;
  end;
  Result := True;
end;
//==================================================================================================
procedure  PSQLCommit( Query: TtbQuery );
begin
  with Query do begin
    SQL.Text := 'COMMIT';
    ExecSQL;
    Close;
  end;
end;
//==================================================================================================
procedure  PSQLRollback( Query: TtbQuery );
begin
  with Query do begin
    SQL.Text := 'ROLLBACK';
    ExecSQL;
    Close;
  end;
end;
//==================================================================================================
procedure BtrieveException( Error: Exception; Operation: String = '' );
  var iError: Integer;
      sError: String;
begin
  if Length( Operation ) = 0 then
    if Length( Application.Hint ) = 0
      then Operation := Application.Title
      else Operation := Application.Hint;
  if Pos( 'Btrieve Status', Error.Message ) > 0 then begin
    sError := Trim( Copy( Error.Message, Pos( 'Btrieve Status', sError ) + 15, 255 ));
    iError := StrToIntDef( Copy ( sError, 1, Pos (' ', sError ) - 1 ), 9999 );
    case iError of
        5:  sError := '  !';
       10:  sError := '  !';
       71:  sError := '  !'#13 +
                      '     ,'#13 +
                      '     .'#13 +
                      '    ,   .';
       84,
       85:  sError := '      .';
       87,
      162:  sError := '   .'#13 +
                      '  ' + IntToStr( iError )+ '.';
      204:  sError := '     .';
      311,
      313:  sError := '   .';
      9999: sError := '   .'#13 + Error.Message;
      else  sError := '   ' + IntToStr( iError ) + '.'
    end;
    Error.Message := sError;
  end;
  ShowStop( Operation + ':'#13 + Error.Message );
end;
//==================================================================================================
procedure SetSettingBeforeQSL;
begin
  DateSeparator   := '/';
  ShortDateFormat := 'mm/dd/yyyy';
  DecimalSeparator := '.'
end;
//==================================================================================================
procedure  RestoreSettingAfterQSL;
begin
  DateSeparator    := sDatSeparat;
  ShortDateFormat  := sShortDatFormat;
  DecimalSeparator := sDecimSeparat;
end;
//==================================================================================================
function  TitanReadPath( Alias: String ): String;
begin
  Result := RegistryGetValue( HKEY_LOCAL_MACHINE,
                              '\SOFTWARE\Titan\Btrieve\Aliases\' + Alias,
                              'PhysicalName',
                              TitanNotPath );
end;
//==================================================================================================
function  TitanReadDescription( Alias: String ): String;
begin
  Result := RegistryGetValue( HKEY_LOCAL_MACHINE,
                              '\SOFTWARE\Titan\Btrieve\Aliases\' + Alias,
                              'Description',
                              TitanNotPath );
end;
//==================================================================================================
procedure TitanWritePath( Alias, Path: String );
begin
  RegistrySetValue( HKEY_LOCAL_MACHINE,
                    '\SOFTWARE\Titan\Btrieve\Aliases\' + Alias,
                    'PhysicalName',
                    Path );
end;
//==================================================================================================
procedure TitanWriteDescription( Alias, Description: String );
begin
  RegistrySetValue( HKEY_LOCAL_MACHINE,
                    '\SOFTWARE\Titan\Btrieve\Aliases\' + Alias,
                    'Description',
                    Description );
end;
//==================================================================================================
initialization  { Initialization section }
  sDatSeparat     := DateSeparator;
  sShortDatFormat := ShortDateFormat;
  sDecimSeparat   := DecimalSeparator;
//==================================================================================================
end.
