{==================================================================================================

  Comming Library

       TQuery.

===================================================================================================

  :

  20.09.2001    Delphi6.
  29.05.2001    CommingBtrieve       
                   TQuery.
               TAB.
                 CommingBtrieveRx.

===================================================================================================
  Copyright 2001 TSK Developer Laboratory, Moscow.                        Konstantin Tsvetkov
                                                                            e-mail: tsklab@mail.ru
===================================================================================================}
unit CommingQuery;

interface

  uses SysUtils, Forms, Db, DBTables, kbmMemTable, Variants;

  function    SQLExecute( Query: TQuery; SQLText: String ): Boolean; overload;
  function    SQLExecute( Query: TQuery; SQLText: String; const Args: array of const ): Boolean; overload;
  function    SQLExecute( SQLDataBase, SQLText: String ): Boolean; overload;
  function    SQLExecute( SQLDataBase, SQLText: String; const Args: array of const ): Boolean; overload;
  function    SQLCalculate( Query: TQuery; SQLText: String ): Variant; overload;
  function    SQLCalculate( Query: TQuery; SQLText: String; const Args: array of const ): Variant; overload;
  function    SQLCalculate( SQLDataBase, SQLText: String ): Variant; overload;
  function    SQLCalculate( SQLDataBase, SQLText: String; const Args: array of const ): Variant; overload;
  function    SQLListing( Query: TQuery; SQLText: String ): Variant; overload;
  function    SQLListing( Query: TQuery; SQLText: String; const Args: array of const ): Variant; overload;
  function    SQLBeginTransaction( Query: TQuery ): Boolean;
  procedure   SQLCommit( Query: TQuery );
  procedure   SQLRollback( Query: TQuery );

  procedure  SQLSelectToMemory( Query: TQuery; SQLText: String;
                                var Data: TkbmMemTable; CopyStructure: Boolean = True ); overload;
  procedure  SQLSelectToMemory( Query: TQuery; SQLText: String; const Args: array of const;
                                var Data: TkbmMemTable; CopyStructure: Boolean = True ); overload;
  procedure  SQLSelectToMemory( SQLDataBase, SQLText: String;
                                var Data: TkbmMemTable; CopyStructure: Boolean = True ); overload;
  procedure  SQLSelectToMemory( SQLDataBase, SQLText: String; const Args: array of const;
                                var Data: TkbmMemTable; CopyStructure: Boolean = True ); overload;
  procedure  SQLRowSelectToMemory( Query: TQuery; SQLText: String;
                                var Data: TkbmMemTable; FieldBegin: Integer = 0 ); overload;
  procedure  SQLRowSelectToMemory( Query: TQuery; SQLText: String; const Args: array of const;
                                var Data: TkbmMemTable; FieldBegin: Integer = 0 ); overload;

implementation
//==================================================================================================
function SQLExecute( Query: TQuery; SQLText: String ): Boolean; overload;
begin
  with Query do begin
    SQL.Text := SQLText;
    ExecSQL;
  end;
  Result := True;
end;
//==================================================================================================
function SQLExecute( Query: TQuery; SQLText: String; const Args: array of const ): Boolean; overload;
begin
  Result := SQLExecute( Query, Format( SQLText, Args ));
end;
//==================================================================================================
function SQLExecute( SQLDataBase, SQLText: String ): Boolean; overload;
  var Query: TQuery;
begin
  Query := TQuery.Create(nil);
  try
    with Query do begin
      DatabaseName := SQLDataBase;
      SQL.Text := SQLText;
      ExecSQL;
    end;
    Result := True;
  finally
    Query.Free;
  end;
end;
//==================================================================================================
function SQLExecute( SQLDataBase, SQLText: String; const Args: array of const ): Boolean; overload;
begin
  Result := SQLExecute( SQLDataBase, Format( SQLText, Args ));
end;
//==================================================================================================
function SQLCalculate( Query: TQuery; SQLText: String ): Variant; overload;
  var i: Integer;
begin
  with Query do begin
    Close;
    SQL.Text := SQLText;
//    SQL.Add( SQLText );
    try
      Open;
      if eof then Result := 0
      else begin
        if Fields.Count > 1 then begin
          Result := VarArrayCreate( [ 0, Fields.Count - 1 ], varVariant );
          for i := 0 to Fields.Count - 1 do begin
            if Fields[i] is TStringField then ( Fields[i] as TStringField ).Transliterate := False;
            Result[i] := Fields[i].AsVariant;
          end;
        end else begin
          if Fields[0] is TStringField then ( Fields[0] as TStringField ).Transliterate := False;
          Result := Fields[0].AsVariant;
        end
      end;
    except on E: Exception do
      if E.Message = 'DBIERR_EOF'
        then Result := 0
        else raise E
    end;
    Close;
  end;
end;
//==================================================================================================
function SQLCalculate( Query: TQuery; SQLText: String; const Args: array of const ): Variant; overload;
begin
  Result := SQLCalculate( Query, Format( SQLText, Args ));
end;
//==================================================================================================
function SQLCalculate( SQLDataBase, SQLText: String ): Variant; overload;
  var Query: TQuery;
      i: Integer;
begin
  Query := TQuery.Create(nil);
  try
    with Query do begin
      DatabaseName := SQLDataBase;
      SQL.Text := SQLText;
      try
      Open;
      if eof then Result := 0
      else begin
        if Fields.Count > 1 then begin
          Result := VarArrayCreate( [ 0, Fields.Count - 1 ], varVariant );
          for i := 0 to Fields.Count - 1 do begin
            if Fields[i] is TStringField then ( Fields[i] as TStringField ).Transliterate := False;
            Result[i] := Fields[i].AsVariant;
          end;
        end else begin
          if Fields[0] is TStringField then ( Fields[0] as TStringField ).Transliterate := False;
          Result := Fields[0].AsVariant;
        end
      end;
      except on E: Exception do
        if E.Message = 'DBIERR_EOF'
          then Result := 0
          else raise
      end;
      Close;
    end;
  finally
    Query.Free;
  end;
end;
//==================================================================================================
function SQLCalculate( SQLDataBase, SQLText: String; const Args: array of const ): Variant; overload;
begin
  Result := SQLCalculate( SQLDataBase, Format( SQLText, Args ));
end;
//==================================================================================================
function SQLListing( Query: TQuery; SQLText: String ): Variant; overload;
  var i: Integer;
begin
  Result := 0;
  with Query do begin
    Close;
    SQL.Text := SQLText;
    try
      Open;
      if RecordCount < 1 then begin
        Result := -1;
        Close;
        Exit;
      end;
      Result := VarArrayCreate([ 1, RecordCount ], varVariant );
      i := 1;
      if Fields[0] is TStringField then ( Fields[0] as TStringField ).Transliterate := False;
      while not eof do begin
        Result[i] := Fields[0].AsVariant;
        Inc( i );
        Next;
      end;
    except on E: Exception do
      if not ( E.Message = 'DBIERR_EOF' ) then raise;
    end;
    Close;
  end;
end;
//==================================================================================================
function SQLListing( Query: TQuery; SQLText: String; const Args: array of const ): Variant; overload;
begin
  Result := SQLListing( Query, Format( SQLText, Args ));
end;
//==================================================================================================
function SQLBeginTransaction( Query: TQuery): Boolean;
begin
//  if not Query.Database.InTransaction then Query.Database.StartTransaction;
  with Query do begin
    SQL.Text := 'BEGIN TRANSACTION';
    ExecSQL;
  end;
  Result := True;
end;
//==================================================================================================
procedure SQLCommit( Query: TQuery );
begin
//  Query.Database.Commit;
  with Query do begin
    SQL.Text := 'COMMIT TRANSACTION';
    ExecSQL;
    Close;
  end;
end;
//==================================================================================================
procedure SQLRollback( Query: TQuery );
begin
//  Query.Database.Rollback;
  with Query do begin
    SQL.Text := 'ROLLBACK TRANSACTION';
    ExecSQL;
    Close;
  end;
end;
//==================================================================================================
procedure CopyRowToMemory( Query: TQuery; var Data: TkbmMemTable; FieldBegin: Integer );
  var i: Integer;
begin
  for i := 0 to Query.Fields.Count - 1 do begin
    if Query.Fields[i].DataType = ftString
      then ( Query.Fields[i] as TStringField ).Transliterate := False;
    Data.Fields[ FieldBegin + i ].Value := Query.Fields[i].Value;
  end;
end;
//==================================================================================================
procedure  CopyQueryToMemory( Query: TQuery; var Data: TkbmMemTable; CopyStructure: Boolean );
  var i: Integer;
begin
  if CopyStructure then begin
    Data.Close;
    Data.CreateTableAs( Query, [ mtcpoStructure ] );
    Data.Open;
  end;
  Query.Last;
  while not Query.Bof do begin
    Data.Insert;
    for i := 0 to Query.Fields.Count - 1 do Data.Fields[i].Value := Query.Fields[i].Value;
    Data.Post;
    Query.Prior;
  end;
end;
//==================================================================================================
procedure  SQLSelectToMemory( Query: TQuery; SQLText: String;
                              var Data: TkbmMemTable; CopyStructure: Boolean = True ); overload;
begin
  with Query do begin
    SQL.Text := SQLText;
    Open;
  end;
  CopyQueryToMemory( Query, Data, CopyStructure );
end;
//==================================================================================================
procedure  SQLSelectToMemory( Query: TQuery; SQLText: String; const Args: array of const;
                              var Data: TkbmMemTable; CopyStructure: Boolean = True ); overload;
begin
  SQLSelectToMemory( Query, Format( SQLText, Args ), Data, CopyStructure );
end;
//==================================================================================================
procedure  SQLSelectToMemory( SQLDataBase, SQLText: String;
                              var Data: TkbmMemTable; CopyStructure: Boolean = True ); overload;
  var Query: TQuery;
begin
  Query := TQuery.Create(nil);
  try
    with Query do begin
      DatabaseName := SQLDataBase;
      SQL.Text := SQLText;
      Open;
    end;
    CopyQueryToMemory( Query, Data, CopyStructure );
  finally
    Query.Free;
  end;
end;
//==================================================================================================
procedure  SQLSelectToMemory( SQLDataBase, SQLText: String; const Args: array of const;
                              var Data: TkbmMemTable; CopyStructure: Boolean = True); overload;
begin
  SQLSelectToMemory( SQLDataBase, Format( SQLText, Args ), Data, CopyStructure );
end;
//==================================================================================================
procedure  SQLRowSelectToMemory( Query: TQuery; SQLText: String;
                              var Data: TkbmMemTable; FieldBegin: Integer = 0 ); overload;
begin
  with Query do begin
    SQL.Text := SQLText;
    Open;
  end;
  CopyRowToMemory( Query, Data, FieldBegin );
end;
//==================================================================================================
procedure  SQLRowSelectToMemory( Query: TQuery; SQLText: String; const Args: array of const;
                              var Data: TkbmMemTable; FieldBegin: Integer = 0 ); overload;
begin
  SQLRowSelectToMemory( Query, Format( SQLText, Args ), Data, FieldBegin );
end;
//==================================================================================================
end.
